@@ -415,11 +415,8 @@ class ZDO(t.CommandsBase, subsystem=t.Subsystem.ZDO):
             t.Param("SrcEndpoint", t.uint8_t, "binding source endpoint"),
             t.Param("ClusterId", t.ClusterId, "Cluster id to match in messages"),
             t.Param(
-                "Address",
-                t.AddrModeAddress,
-                "Binding address mode destination address",
+                "Address", zigpy.zdo.types.MultiAddress, "Unbinding address/endpoint"
             ),
-            t.Param("DstEndpoint", t.uint8_t, "Binding destination endpoint"),
         ),
         rsp_schema=t.STATUS_SCHEMA,
     )
