@@ -41,12 +41,14 @@ with warnings.catch_warnings():
 
 
 ZDO_ENDPOINT = 0
+
 PROBE_TIMEOUT = 5  # seconds
 STARTUP_TIMEOUT = 5  # seconds
 ZDO_REQUEST_TIMEOUT = 15  # seconds
 DATA_CONFIRM_TIMEOUT = 8  # seconds
 DEVICE_JOIN_MAX_DELAY = 2  # seconds
 NETWORK_COMMISSIONING_TIMEOUT = 30  # seconds
+WATCHDOG_PERIOD = 30  # seconds
 
 REQUEST_MAX_RETRIES = 5
 REQUEST_ERROR_RETRY_DELAY = 0.5  # second
@@ -91,15 +93,12 @@ class ZNPCoordinator(zigpy.device.Device):
 
     @property
     def model(self):
-        # There is no way to query the Z-Stack version or even the hardware at runtime.
-        # Instead we have to rely on these sorts of "hints"
-        if self.application.is_cc2531:
+        if self.application._znp.version == 1.2:
             model = "CC2531"
-
-            if self.application.is_zstack_home_12:
-                version = "Home 1.2"
-            else:
-                version = "3.0.1/3.0.2"
+            version = "Home 1.2"
+        elif self.application._znp.version == 3.0:
+            model = "CC2531"
+            version = "3.0.1/3.0.2"
         else:
             model = "CC13X2/CC26X2"
             version = "3.30.00/3.40.00/4.10.00"
@@ -120,8 +119,12 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         self._reconnect_task = asyncio.Future()
         self._reconnect_task.cancel()
 
+        self._watchdog_task = asyncio.Future()
+        self._watchdog_task.cancel()
+
         self._nib = NIB()
         self._network_key = None
+        self._network_key_seq = None
         self._concurrent_requests_semaphore = None
         self._currently_waiting_requests = 0
         self._route_discovery_futures = {}
@@ -131,27 +134,16 @@ class ControllerApplication(zigpy.application.ControllerApplication):
     # Implementation of the core zigpy ControllerApplication methods #
     ##################################################################
 
-    @property
-    def channel(self) -> typing.Optional[int]:
-        return self._nib.nwkLogicalChannel
-
-    @property
-    def channels(self) -> typing.Optional[t.Channels]:
-        return self._nib.channelList
-
-    @property
-    def pan_id(self) -> typing.Optional[t.NWK]:
-        return self._nib.nwkPanId
-
-    @property
-    def ext_pan_id(self) -> typing.Optional[t.EUI64]:
-        return self._nib.extendedPANID
-
     @property
     def network_key(self) -> typing.Optional[t.KeyData]:
         # This is not a standard Zigpy property
         return self._network_key
 
+    @property
+    def network_key_seq(self) -> typing.Optional[t.uint8_t]:
+        # This is not a standard Zigpy property
+        return self._network_key_seq
+
     @classmethod
     async def probe(cls, device_config: conf.ConfigType) -> bool:
         """
@@ -166,35 +158,37 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             async with async_timeout.timeout(PROBE_TIMEOUT):
                 await znp.connect()
 
-            result = True
+            return True
         except Exception as e:
-            result = False
-            LOGGER.warning(
+            LOGGER.debug(
                 "Failed to probe ZNP radio with config %s", device_config, exc_info=e
             )
-
-        znp.close()
-
-        return result
+            return False
+        finally:
+            znp.close()
 
     async def shutdown(self):
         """
         Gracefully shuts down the application and cleans up all resources.
-        This method calls ZNP.close, which calls UART.close, etc.
         """
 
+        self.close()
+
+    def close(self):
         self._reconnect_task.cancel()
+        self._watchdog_task.cancel()
 
         for f in self._route_discovery_futures.values():
             f.cancel()
 
         self._route_discovery_futures.clear()
 
+        # This will close the UART, which will then close the transport
         if self._znp is not None:
             self._znp.close()
             self._znp = None
 
-    async def startup(self, auto_form=False):
+    async def startup(self, auto_form=False, force_form=False):
         """
         Performs application startup.
 
@@ -203,12 +197,12 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         """
 
         try:
-            return await self._startup(auto_form=auto_form)
+            return await self._startup(auto_form=auto_form, force_form=force_form)
         except Exception:
             await self.shutdown()
             raise
 
-    async def _startup(self, auto_form=False):
+    async def _startup(self, auto_form=False, force_form=False):
         assert self._znp is None
 
         znp = ZNP(self.config)
@@ -222,7 +216,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         # Next, read out the NVRAM item that Zigbee2MQTT writes when it has configured
         # a device to make sure that our network settings will not be reset.
-        if self.is_zstack_home_12:
+        if self._znp.version == 1.2:
             configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK1
         else:
             configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK3
@@ -233,8 +227,8 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         except KeyError:
             is_configured = False
 
-        if not is_configured:
-            if not auto_form:
+        if not is_configured or force_form:
+            if not auto_form and not force_form:
                 raise RuntimeError("Cannot start application, network is not formed")
 
             LOGGER.info("ZNP is not configured, forming a new network")
@@ -264,7 +258,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         )
 
         # The AUTOSTART startup NV item doesn't do anything.
-        if self.is_zstack_home_12:
+        if self._znp.version == 1.2:
             # Z-Stack Home 1.2 has a simple startup sequence
             await self._znp.request(
                 c.ZDO.StartupFromApp.Req(StartDelay=100),
@@ -300,17 +294,12 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         # XXX: The CC2531 running Z-Stack Home 1.2 permanently permits joins on startup
         # unless they are explicitly disabled. We can't fix this but we can disable them
         # as early as possible to shrink the window of opportunity for unwanted joins.
-        await self.permit_ncp(time_s=0)
+        await self.permit(time_s=0)
 
         # The CC2531 running Z-Stack Home 1.2 overrides the LED setting if it is changed
         # before the coordinator has started.
         if self.znp_config[conf.CONF_LED_MODE] is not None:
-            led_mode = self.znp_config[conf.CONF_LED_MODE]
-
-            await self._znp.request(
-                c.Util.LEDControl.Req(LED=0xFF, Mode=led_mode),
-                RspStatus=t.Status.SUCCESS,
-            )
+            await self._set_led_mode(led=0xFF, mode=self.znp_config[conf.CONF_LED_MODE])
 
         device_info = await self._znp.request(
             c.Util.GetDeviceInfo.Req(), RspStatus=t.Status.SUCCESS
@@ -359,12 +348,21 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         self._concurrent_requests_semaphore = asyncio.Semaphore(max_concurrent_requests)
 
-        LOGGER.info(
-            "Currently on channel %d. Limiting concurrent requests to %d",
-            self.channel,
-            max_concurrent_requests,
+        version_rsp = await self._znp.request(c.SYS.Version.Req())
+
+        LOGGER.info("Network settings")
+        LOGGER.info("  Z-Stack version: %s", self._znp.version)
+        LOGGER.info("  Z-Stack build id: %s", version_rsp.CodeRevision)
+        LOGGER.info("  Max concurrent requests: %s", max_concurrent_requests)
+        LOGGER.info("  Channel: %s", self.channel)
+        LOGGER.info("  PAN ID: 0x%04x", self.pan_id)
+        LOGGER.info("  Extended PAN ID: %s", self.extended_pan_id)
+        LOGGER.debug(
+            "  Network key: %s", ":".join(f"{c:02x}" for c in self.network_key)
         )
 
+        self._watchdog_task = asyncio.create_task(self._watchdog_loop())
+
     async def update_network_channel(self, channel: t.uint8_t):
         """
         Changes the network channel, increments the beacon update ID, and waits until
@@ -393,6 +391,19 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             await self._load_device_info()
             await asyncio.sleep(1)
 
+    async def update_pan_id(self, pan_id: t.uint16_t) -> None:
+        """
+        Updates the network PAN ID, bypassing Z-Stack anti-collision checks.
+        """
+
+        await self._reset()
+
+        nib = parse_nib(await self._znp.nvram.osal_read(OsalNvIds.NIB))
+        nib.nwkPanId = pan_id
+
+        await self._znp.nvram.osal_write(OsalNvIds.NIB, nib)
+        await self._znp.nvram.osal_write(OsalNvIds.PANID, pan_id)
+
     async def form_network(self):
         """
         Clears the current config and forms a new network with a random network key,
@@ -440,7 +451,11 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         LOGGER.debug("Updating network settings")
 
         await self._write_stack_settings(reset_if_changed=False)
-        await self._znp.nvram.osal_write(OsalNvIds.PANID, pan_id, create=True)
+
+        # Ignore the user's PAN ID for now and just let Z-Stack pick one that works
+        await self._znp.nvram.osal_write(
+            OsalNvIds.PANID, t.uint16_t(0xFFFF), create=True
+        )
         await self._znp.nvram.osal_write(
             OsalNvIds.APS_USE_EXT_PANID, extended_pan_id, create=True
         )
@@ -451,7 +466,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         await self._znp.nvram.osal_write(OsalNvIds.CHANLIST, channels, create=True)
 
         # Z-Stack Home 1.2 doesn't have the BDB subsystem
-        if not self.is_zstack_home_12:
+        if self._znp.version > 1.2:
             await self._znp.request(
                 c.AppConfig.BDBSetChannel.Req(IsPrimary=True, Channel=channels),
                 RspStatus=t.Status.SUCCESS,
@@ -482,7 +497,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
                 ),
             ]
         ):
-            if not self.is_zstack_home_12:
+            if self._znp.version > 1.2:
                 # Z-Stack 3 uses the BDB subsystem
                 commissioning_rsp = await self._znp.request_callback_rsp(
                     request=c.AppConfig.BDBStartCommissioning.Req(
@@ -536,8 +551,13 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
             await asyncio.sleep(1)
 
+        if pan_id != 0xFFFF:
+            # Update the PAN ID at runtime after the network has formed to make sure
+            # Z-Stack does not complain about collisions on Z-Stack 3.
+            await self.update_pan_id(pan_id)
+
         # Create the NV item that keeps track of whether or not we're fully configured.
-        if self.is_zstack_home_12:
+        if self._znp.version == 1.2:
             configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK1
         else:
             configured_nv_item = OsalNvIds.HAS_CONFIGURED_ZSTACK3
@@ -659,44 +679,23 @@ class ControllerApplication(zigpy.application.ControllerApplication):
     async def force_remove(self, device: zigpy.device.Device) -> None:
         """
         Attempts to forcibly remove a device from the network.
-        Z-Stack does not expose any additional ways to to do this.
         """
 
-        try:
-            async with self._limit_concurrency():
-                await self._znp.request_callback_rsp(
-                    request=c.ZDO.MgmtLeaveReq.Req(
-                        DstAddr=0x0000,
-                        IEEE=device.ieee,
-                        RemoveChildren_Rejoin=c.zdo.LeaveOptions.NONE,
-                    ),
-                    RspStatus=t.Status.SUCCESS,
-                    callback=c.ZDO.MgmtLeaveRsp.Callback(
-                        Src=0x0000,
-                        partial=True,
-                    ),
-                )
-        except Exception as e:
-            LOGGER.warning(
-                "Failed to remove device %s from coordinator: %s",
-                device.ieee,
-                e,
-            )
+        LOGGER.warning("Z-Stack does not support force remove")
 
-    async def permit_ncp(self, time_s: int) -> None:
+    async def permit(self, time_s=60, node=None):
         """
-        Permits new devices to join the network *only through us*.
-        Zigpy sends a broadcast to all routers on its own.
+        Permit joining the network via a specific node or via all router nodes.
         """
 
+        # Always permit joins on the coordinator first.
+        # This unfortunately makes it impossible to permit joins via just one router
+        # alone but firmware changes can make this possible on newer hardware.
         response = await self._znp.request_callback_rsp(
             request=c.ZDO.MgmtPermitJoinReq.Req(
                 AddrMode=t.AddrMode.NWK,
-                Dst=0x0000,  # Only us!
+                Dst=0x0000,
                 Duration=time_s,
-                # "This field shall always have a value of 1,
-                #  indicating a request to change the
-                #  Trust Center policy."
                 TCSignificance=1,
             ),
             RspStatus=t.Status.SUCCESS,
@@ -704,7 +703,17 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         )
 
         if response.Status != t.Status.SUCCESS:
-            raise RuntimeError(f"Permit join response failure: {response}")
+            raise RuntimeError(f"Failed to permit joins on the coordinator: {response}")
+
+        return await super().permit(time_s=time_s, node=node)
+
+    async def permit_ncp(self, time_s: int) -> None:
+        """
+        Permits joins only on the coordinator.
+        """
+
+        # Z-Stack does not need any special code to do this
+        pass
 
     async def permit_with_key(self, node: t.EUI64, code: bytes, time_s=60):
         """
@@ -753,17 +762,12 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         Spawns the auto-reconnect task.
         """
 
-        self._znp = None
+        LOGGER.debug("Connection lost: %s", exc)
 
-        # exc=None means the connection was closed
-        if exc is None:
-            LOGGER.debug("Connection was purposefully closed. Not reconnecting.")
-            return
+        self.close()
 
-        # Reconnect in the background using our previously-detected port.
-        if self._reconnect_task.done():
-            LOGGER.debug("Restarting background reconnection task")
-            self._reconnect_task = asyncio.create_task(self._reconnect())
+        LOGGER.debug("Restarting background reconnection task")
+        self._reconnect_task = asyncio.create_task(self._reconnect())
 
     #####################################################
     # Z-Stack message callbacks attached during startup #
@@ -800,6 +804,20 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             c.ZDO.SrcRtgInd.Callback(partial=True), self.on_zdo_relays_message
         )
 
+        self._znp.callback_for_response(
+            c.ZDO.PermitJoinInd.Callback(partial=True), self.on_zdo_permit_join_message
+        )
+
+    def on_zdo_permit_join_message(self, msg: c.ZDO.PermitJoinInd.Callback) -> None:
+        """
+        Coordinator join status change message. Only sent with Z-Stack 1.2 and 3.0.
+        """
+
+        if msg.Duration == 0:
+            LOGGER.info("Coordinator is not permitting joins anymore")
+        else:
+            LOGGER.info("Coordinator is permitting joins for %d seconds", msg.Duration)
+
     def on_zdo_relays_message(self, msg: c.ZDO.SrcRtgInd.Callback) -> None:
         """
         ZDO source routing message callback
@@ -921,14 +939,6 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         return self.devices[self.ieee]
 
-    @property
-    def is_zstack_home_12(self) -> bool:
-        return self._znp._version.MinorRel == 6
-
-    @property
-    def is_cc2531(self) -> bool:
-        return isinstance(self._nib, CC2531NIB)
-
     @property
     def znp_config(self) -> conf.ConfigType:
         """
@@ -937,6 +947,50 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         return self.config[conf.CONF_ZNP_CONFIG]
 
+    async def _watchdog_loop(self):
+        """
+        Watchdog loop to periodically test if Z-Stack is still running.
+        """
+
+        LOGGER.debug("Starting watchdog loop")
+
+        while True:
+            await asyncio.sleep(WATCHDOG_PERIOD)
+
+            # No point in trying to test the port if it's already disconnected
+            if self._znp is None:
+                break
+
+            try:
+                await self._znp.request(c.SYS.Ping.Req())
+            except Exception as e:
+                LOGGER.error(
+                    "Watchdog check failed",
+                    exc_info=e,
+                )
+
+                # Treat the watchdog failure as a disconnect
+                self.connection_lost(e)
+
+                return
+
+    async def _set_led_mode(self, *, led, mode) -> None:
+        """
+        Attempts to set the provided LED's mode. A Z-Stack bug causes the underlying
+        command to never receive a response if the board has no LEDs, requiring this
+        wrapper function to prevent the command from taking many seconds to time out.
+        """
+
+        # XXX: If Z-Stack is not compiled with HAL_LED, it will just not respond at all
+        try:
+            async with async_timeout.timeout(0.3):
+                await self._znp.request(
+                    c.Util.LEDControl.Req(LED=led, Mode=mode),
+                    RspStatus=t.Status.SUCCESS,
+                )
+        except (asyncio.TimeoutError, CommandNotRecognized):
+            LOGGER.info("This build of Z-Stack does not appear to support LED control")
+
     async def _write_stack_settings(self, *, reset_if_changed: bool) -> None:
         """
         Writes network-independent Z-Stack settings to NVRAM.
@@ -982,6 +1036,11 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         Mainly a thin wrapper around `asyncio.Semaphore` that logs when it has to wait.
         """
 
+        # Allow sending some requests before the application has fully started
+        if self._concurrent_requests_semaphore is None:
+            yield
+            return
+
         start_time = time.time()
         was_locked = self._concurrent_requests_semaphore.locked()
 
@@ -1055,8 +1114,15 @@ class ControllerApplication(zigpy.application.ControllerApplication):
             try:
                 await self._startup()
                 return
+            except asyncio.CancelledError:
+                raise
             except Exception as e:
                 LOGGER.error("Failed to reconnect", exc_info=e)
+
+                if self._znp is not None:
+                    self._znp.close()
+                    self._znp = None
+
                 await asyncio.sleep(
                     self._config[conf.CONF_ZNP_CONFIG][
                         conf.CONF_AUTO_RECONNECT_RETRY_DELAY
@@ -1107,15 +1173,32 @@ class ControllerApplication(zigpy.application.ControllerApplication):
         )
 
     async def _load_device_info(self):
+        """
+        Loads low-level network information from NVRAM.
+        """
+
         # Parsing the NIB struct gives us access to low-level info, like the channel
         self._nib = parse_nib(await self._znp.nvram.osal_read(OsalNvIds.NIB))
-        self._network_key, _ = t.KeyData.deserialize(
-            await self._znp.nvram.osal_read(OsalNvIds.PRECFGKEY)
-        )
-
         LOGGER.debug("Parsed NIB: %s", self._nib)
 
-    async def _reset(self):
+        nwkkey = await self._znp.nvram.osal_read(OsalNvIds.NWKKEY)
+
+        if self._znp.version < 3.30:
+            key_info, _ = t.NwkActiveKeyItemsCC2531.deserialize(nwkkey)
+        else:
+            key_info, _ = t.NwkActiveKeyItems.deserialize(nwkkey)
+
+        self._channel = self._nib.nwkLogicalChannel
+        self._channels = self._nib.channelList
+        self._pan_id = self._nib.nwkPanId
+        self._ext_pan_id = self._nib.extendedPANID
+
+        self._network_key = key_info.Active.Key
+        self._network_key_seq = key_info.Active.KeySeqNum
+
+        LOGGER.debug("Parsed key info: %s", key_info)
+
+    async def _reset(self) -> None:
         """
         Performs a soft reset within Z-Stack.
         A hard reset resets the serial port, causing the device to disconnect.
@@ -1330,7 +1413,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
 
         # Route discovery with Z-Stack 1.2 and Z-Stack 3.0.2 on the CC2531 doesn't
         # appear to work very well (Z2M#2901)
-        if self.is_cc2531:
+        if self._znp.version < 3.30:
             return
 
         if nwk in self._route_discovery_futures:
@@ -1464,6 +1547,7 @@ class ControllerApplication(zigpy.application.ControllerApplication):
                             status == t.Status.MAC_TRANSACTION_EXPIRED
                             and association is None
                             and not tried_assoc_remove
+                            and self._znp.version >= 3.30
                         ):
                             # XXX: do we use NWK or IEEE?
                             association = await self._znp.request(
@@ -1501,15 +1585,11 @@ class ControllerApplication(zigpy.application.ControllerApplication):
                             # generate a bit more network traffic.
                             options &= ~c.af.TransmitOptions.SUPPRESS_ROUTE_DISC_NETWORK
                             tried_disable_route_discovery_suppression = True
-                        elif (
-                            not tried_last_good_route
-                            and device is not None
-                            and device.relays is not None
-                        ):
+                        elif not tried_last_good_route and device is not None:
                             # `ZDO.SrcRtgInd` callbacks tell us the last path taken by
                             # messages from the device back to the coordinator. Sending
                             # packets backwards via this same route may work.
-                            force_relays = device.relays[::-1]
+                            force_relays = (device.relays or [])[::-1]
                             tried_last_good_route = True
 
                         LOGGER.debug(
